@@ -34,11 +34,11 @@ local.file.exists <- function(fileName) {
 
 #' Check if the inputs are valid tabix range such as chr1:2-300
 #' @param range characer vector
-#' @keywords internal
+#' @export
 #' @examples
-#' valid <- isTabixRange(c("chr1:1-200", "X:1", "1:100-100", "chr1", "1:1-20,1:30-40))
+#' valid <- isTabixRange(c("chr1:1-200", "X:1", "1:100-100", "chr1", "1:1-20,1:30-40"))
 #' stopifnot(all(valid))
-#' invalid <- isTabixRange(c(":1", "chr1::"":-"))
+#' invalid <- isTabixRange(c(":1", "chr1::", ":-"))
 #' stopifnot(all(!invalid))
 isTabixRange <- function(range) {
   isValid <- function(x) {
@@ -70,10 +70,8 @@ isTabixRange <- function(range) {
     }
     return(TRUE)
   }
-  input <- strsplit(range, ",")
-  do.call(c, lapply(input, function(x){
-    isValid(x)
-    }))
+  ranges <- strsplit(x = range, split = ",")[[1]]
+  sapply(ranges, isValid)
 }
 
 #' Check input file has tabix index
